%Smoothing optimization for Indefinite SVM, proposed  and coded by Yiming,
%May, 2009
function  [alpha,history] = IndefiniteSVM_nes(K,Y,C, rho, L, tol, maxiters,display)
%Inputs:
%K -- indefinite kernel matrix
%Y -- labels (binary)
%rho -- regularization parameter for psd kernel matrix
%C -- trade-off parameter for SVM
%tol --- torelance for stopping criterion
%L -- Lipschitz constant for gradient fucntion
%maxters --- maximum iterations
% calling eigUpdateMult_mex.m  and projcutsimplex4, so  first make sure
% that you install IndefiniteSVM package by Roth Luss and A. d'Aspremont (NIPS-2007)
% Copyright (C) 2009-2010, Yiming Ying.
% You are free to use this software for academic purpose, however, without any of my guarantees.

ntr = length(Y);[V0,D0]=eig(K);D0=diag(real(D0));
weight=sum(Y>0)/sum(Y<0); % weight the SVM C penalties for over/under representation
C_weighted=weightPenalty(C,Y,weight);
alpha_0=projcutsimplex4(C_weighted/2,Y,C_weighted,0); % compute an initial point
 alpha =alpha_0;

accu_gradf = zeros(ntr,1);
nonconverged = 1;


its =0;
time0 = cputime; 
objfvalue=[];
dualvalue=[];
changegap = [];
 
fprintf('--Starting Smoothing Optimization for Indefinite SVM\n');

while (nonconverged) && its <maxiters

    % compute projection to psd cone and gradient function
    labls_alpha = Y.*alpha;
    [V,D]=eigUpdateMult_mex(V0,D0,V0'*labls_alpha,1/(4*rho));
    D= (real(D)); V = real(V);

    if sum(sum(isnan([V,D])))>0 % for numerical stability
        [V,D]=eig(K+labls_alpha*labls_alpha'/(4*rho));D=diag(real(D)); V = real(V);
        %         disp(['eigUpateMult_mex failed at iteration ',num2str(iter),' with C=',num2str(c)]);
    end
    tind = find(D>0); 
    Kast = V(:,tind)*diag(D(tind))*V(:,tind)';
    temp = - sum(alpha) + 0.5* labls_alpha'*Kast*labls_alpha - rho*sum(sum((Kast-K).^2));
    objfvalue= [objfvalue -temp];

    %compute the dual

    %     svmmodel = svmtrain(Y,[(1:length(Y))',Kast],libsvmparam);
    %     alpha_temp=zeros(length(Y),1);alpha_temp(svmmodel.SVs)=abs(svmmodel.sv_coef);
    %     dualvalue=[dualvalue; sum(alpha_temp)-0.5*alpha_temp'*diag(Y)*Kast*diag(Y)*alpha_temp...
    %         + rho*trace((Kast-K)'*(Kast-K))];

    %dualgap=[dualgap;  dualvalue(end)-objfvalue(end)];

    gradf = - ones(ntr,1) + diag(Y)*Kast*labls_alpha ;
    accu_gradf = accu_gradf + 0.5*(its+1).*gradf;

    %gradient mapping  (projection)
    gak = projcutsimplex4(alpha-gradf/L,Y,C.*ones(ntr,1),0);

    %updated from accumulate gradient information  (projcetion)
    gbk = projcutsimplex4(alpha_0-accu_gradf/L,Y,C.*ones(ntr,1),0);

    %update alpha
    alpha = (2.*gbk + (its+1).*gak)./(its+3);
    its = its+1;
    if  its>2
        changegap = [changegap, abs(objfvalue(end)-objfvalue(end-1))/abs(objfvalue(end))];
        if display==1
            fprintf('Iter %d   | Primal  %f   |change %f |\n', its,objfvalue(end),...
                changegap(end));
        end
        if  (changegap(end)<tol)
            nonconverged =0;
        end
    end
end
 fprintf('--End of Smoothing Optimization for Indefinite SVM\n');
history.objfvalue = objfvalue;
history.dualvalue = dualvalue;
history.time = cputime-time0;



